# -*- coding: utf-8; -*-
"""
Bootstrap development for Theo
"""

import os
import subprocess
import sys


here = os.path.abspath(os.path.dirname(__file__))


def bootstrap():
    if not inside_virtualenv():
        return

    # install wheel
    subprocess.run(['pip', 'install', 'wheel'],
                   check=True)

    # install invoke, sphinx
    subprocess.run(['pip', 'install', 'invoke', 'Sphinx'],
                   check=True)

    # run bootstrap task
    os.chdir(here)
    try:
        if sys.platform == 'win32':
            completed = subprocess.run(['invoke', 'bootstrap'])
        else:
            completed = subprocess.run(['invoke', '--echo', 'bootstrap'])
    except KeyboardInterrupt:
        sys.exit(130)  # 128 + SIGINT
    else:
        sys.exit(completed.returncode)


def inside_virtualenv():
    if not (hasattr(sys, 'real_prefix') or
            (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix)):
        print("")
        print("Not running inside a virtual environment!")
        print("")
        print("Please create and activate that first, e.g. like:")
        print("")
        if sys.platform == 'win32':
            print("    py -m venv C:\\envs\\theo")
            print("    C:\\envs\\theo\\Scripts\\activate.bat")
        else:
            print("    python -m venv /srv/envs/theo")
            print("    source /srv/envs/theo/bin/activate")
        print("")
        return False
    return True


if __name__ == '__main__':
    bootstrap()



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Views
"""

from tailbone.views import typical

from theo.config import integrate_catapult, integrate_corepos, integrate_locsms


def includeme(config):
    rattail_config = config.registry.settings.get('rattail_config')

    config.include('tailbone.views.essentials')
    config.include('tailbone.views.messages')
    config.include('tailbone.views.trainwreck.defaults')

    mods = {}

    if integrate_catapult(rattail_config):
        config.include('tailbone_onager.views')
        config.include('tailbone_onager.views.catapult')

    elif integrate_corepos(rattail_config):
        config.include('tailbone_corepos.views')
        config.include('tailbone_corepos.views.corepos')
        mods['tailbone.views.purchases'] = 'tailbone_corepos.views.purchases'

    elif integrate_locsms(rattail_config):
        config.include('tailbone_locsms.views')
        config.include('tailbone_locsms.views.locsms')

    typical.defaults(config, **mods)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2024 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Static assets
"""

from fanstatic import Library, Resource


# libcache
libcache = Library('theo_libcache', 'libcache')
bb_vue_js = Resource(libcache, 'vue.esm-browser-3.4.31.prod.js')
bb_oruga_js = Resource(libcache, 'oruga-0.8.12.js')
bb_oruga_bulma_js = Resource(libcache, 'oruga-bulma-0.3.0.js')
bb_oruga_bulma_css = Resource(libcache, 'oruga-bulma-0.3.0.css')
bb_fontawesome_svg_core_js = Resource(libcache, 'fontawesome-svg-core-6.5.2.js')
bb_free_solid_svg_icons_js = Resource(libcache, 'free-solid-svg-icons-6.5.2.js')
bb_vue_fontawesome_js = Resource(libcache, 'vue-fontawesome-3.0.6.index.es.js')


def includeme(config):
    config.include('tailbone.static')
    config.add_static_view('tailbone_theo', 'tailbone_theo.web:static', cache_max_age=3600)



# -*- coding: utf-8; -*-
"""
Development Tasks for Theo
"""

import os
import configparser
import shutil
import sys

from invoke import task
from sphinx.util.console import bold
from sphinx.cmd.quickstart import do_prompt

from bootstrap import inside_virtualenv


here = os.path.abspath(os.path.dirname(__file__))


@task
def bootstrap(c):
    """
    Bootstrap a development environment.
    """
    if not inside_virtualenv():
        sys.exit(1)

    envdir = sys.prefix
    upgrade_pip(c)
    install_app_package(c)
    appdir = make_appdir(c, envdir)

    info = collect_info()
    make_configs(c, envdir, appdir, info)
    check_db(c, envdir, appdir)
    install_db_schema(c, envdir, appdir)
    make_admin_user(c, envdir, appdir, info)

    print()
    print(bold("============================================================"))
    print()
    print(bold("  Okay, you should be ready to go!"))
    print()
    print(bold("============================================================"))
    print()
    print("start your development web app with this command:")
    print()
    print("    cd {}".format(envdir))
    if sys.platform == 'win32':
        print(r"    Scripts\pserve --reload file+ini:app\web.conf")
    else:
        print("    bin/pserve --reload file+ini:app/web.conf")
    print()
    print("then check out your development web app at:")
    print()
    print("    http://localhost:9080")
    print()


def collect_info():
    """
    Collect misc. dev info from user
    """
    info = {}
    print()
    print(bold("Welcome to Theo, the order system."))

    config = configparser.ConfigParser()
    if config.read(os.path.join(here, 'settings.ini')):
        if config.has_section('devbootstrap'):
            info = dict(config.items('devbootstrap'))

    try:
        print()
        print("Please enter the details for your Theo database.")
        print()
        info['dbhost'] = do_prompt('DB host',
                                   default=info.get('dbhost', 'localhost'))
        info['dbname'] = do_prompt('DB name',
                                   default=info.get('dbname', 'theo'))
        info['dbuser'] = do_prompt('DB user',
                                   default=info.get('dbuser', 'rattail'))
        info['dbpass'] = do_prompt('DB password',
                                   default=info.get('dbpass'))

        print()
        print("Please enter the details for your Theo admin user.")
        print()
        info['theouser'] = do_prompt('Theo username',
                                     default=info.get('theouser', 'admin'))
        info['theopass'] = do_prompt('Theo password',
                                     default=info.get('theopass'))

    except (KeyboardInterrupt, EOFError):
        print("\n[Interrupted.]")
        sys.exit(130)  # 128 + SIGINT

    return info


def upgrade_pip(c):
    """
    Upgrade pip and friends
    """
    if sys.platform == 'win32':
        c.run('python -m pip install -U pip')
    else:
        c.run('pip install -U pip')
    c.run('pip install -U setuptools wheel')


def install_app_package(c):
    """
    Install the Theo app package
    """
    project = os.path.abspath(os.path.join(here, os.pardir))
    c.run('pip install -e {}'.format(project))
    c.run('pip install tailbone-theo[app]')


def make_appdir(c, envdir):
    """
    Create the 'app' dir for virtual env
    """
    appdir = os.path.join(envdir, 'app')
    if not os.path.exists(appdir):
        if sys.platform == 'win32':
            c.run('{} make-appdir --path {}'.format(
                os.path.join(envdir, 'Scripts', 'rattail'),
                appdir))
        else:
            c.run('{}/bin/rattail make-appdir --path {}'.format(
                envdir, appdir))
    return appdir


def make_configs(c, envdir, appdir, info):
    """
    Create app config files
    """
    # rattail.conf
    if not os.path.exists(os.path.join(appdir, 'rattail.conf')):
        with open('rattail.conf') as f:
            contents = f.read()
        contents = contents.replace('<ENVDIR>', envdir)
        contents = contents.replace('<SEP>', os.sep)
        contents = contents.replace('<DBHOST>', info['dbhost'])
        contents = contents.replace('<DBNAME>', info['dbname'])
        contents = contents.replace('<DBUSER>', info['dbuser'])
        contents = contents.replace('<DBPASS>', info['dbpass'])
        with open(os.path.join(appdir, 'rattail.conf'), 'w') as f:
            f.write(contents)

    # quiet.conf
    if not os.path.exists(os.path.join(appdir, 'quiet.conf')):
        if sys.platform == 'win32':
            c.run('{} make-config -T quiet -O {}'.format(
                  os.path.join(envdir, 'Scripts', 'rattail'),
                  appdir))
        else:
            c.run('{}/bin/rattail make-config -T quiet -O {}'.format(
                envdir, appdir))

    # nodb.conf
    if not os.path.exists(os.path.join(appdir, 'nodb.conf')):
        shutil.copyfile('nodb.conf', os.path.join(appdir, 'nodb.conf'))

    # web.conf
    if not os.path.exists(os.path.join(appdir, 'web.conf')):
        with open('web.conf') as f:
            contents = f.read()
        contents = contents.replace('<ENVDIR>', envdir)
        contents = contents.replace('<SEP>', os.sep)
        with open(os.path.join(appdir, 'web.conf'), 'w') as f:
            f.write(contents)


def check_db(c, envdir, appdir):
    """
    Do basic sanity checks for Theo database
    """
    if sys.platform == 'win32':
        c.run('{} -c {} -c {} --no-versioning checkdb'.format(
            os.path.join(envdir, 'Scripts', 'rattail'),
            os.path.join(appdir, 'quiet.conf'),
            os.path.join(appdir, 'nodb.conf')))
    else:
        c.run('{0}/bin/rattail -c {1}/quiet.conf -c {1}/nodb.conf --no-versioning checkdb'.format(
            envdir, appdir))


def install_db_schema(c, envdir, appdir):
    """
    Install the schema for Theo database
    """
    if sys.platform == 'win32':
        c.run('{} -c {} upgrade heads'.format(
              os.path.join(envdir, 'Scripts', 'alembic'),
              os.path.join(appdir, 'rattail.conf')))
    else:
        c.run('{}/bin/alembic -c {}/rattail.conf upgrade heads'.format(
            envdir, appdir))


def make_admin_user(c, envdir, appdir, info):
    """
    Make an admin user in the Theo database
    """
    if sys.platform == 'win32':
        c.run('{} -c {} make-user --admin {} --password {}'.format(
              os.path.join(envdir, 'Scripts', 'rattail'),
              os.path.join(appdir, 'quiet.conf'),
              info['theouser'], info['theopass']))
    else:
        c.run('{}/bin/rattail -c {}/quiet.conf make-user --admin {} --password {}'.format(
            envdir, appdir, info['theouser'], info['theopass']))



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Theo web app
"""

from sqlalchemy.orm import sessionmaker, scoped_session
from zope.sqlalchemy import register

from tailbone import app

from theo.config import integrate_catapult, integrate_corepos, integrate_locsms


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    """
    # establish rattail config
    rattail_config = app.make_rattail_config(settings)

    # which template directories we want depends on integration
    directories = ['theo.web:templates']
    if integrate_corepos(rattail_config):
        directories.append('tailbone_corepos:templates')
    if integrate_catapult(rattail_config):
        directories.append('tailbone_onager:templates')
    if integrate_locsms(rattail_config):
        directories.append('tailbone_locsms:templates')
    directories.append('tailbone:templates')
    settings.setdefault('mako.directories', directories)

    # for graceful handling of postgres restart
    settings.setdefault('retry.attempts', 2)

    # establish pyramid config
    pyramid_config = app.make_pyramid_config(settings)

    # Catapult DB(s)
    if integrate_catapult(rattail_config):
        from tailbone_onager.db import CatapultSession, ExtraCatapultSessions

        # default session / engine
        CatapultSession.configure(bind=rattail_config.catapult_engine)

        # make session wrappers for each "extra" engine
        for key, engine in rattail_config.catapult_engines.items():
            if key != 'default':
                Session = scoped_session(sessionmaker(bind=engine))
                register(Session)
                ExtraCatapultSessions[key] = Session

    # LOC SMS DB(s)
    if integrate_locsms(rattail_config):
        from tailbone_locsms.db import SmsSession
        SmsSession.configure(bind=rattail_config.locsms_engine)

    # bring in the rest of Theo
    pyramid_config.include('theo.web.static')
    pyramid_config.include('theo.web.subscribers')
    pyramid_config.include('theo.web.views')

    return pyramid_config.make_wsgi_app()


def asgi_main():
    """
    This function returns an ASGI application.
    """
    from tailbone.asgi import make_asgi_app

    return make_asgi_app(main)
